/* Copyright (C) 1996 Aladdin Enterprises.  All rights reserved.
  
  This file is part of Aladdin Ghostscript.
  
  Aladdin Ghostscript is distributed with NO WARRANTY OF ANY KIND.  No author
  or distributor accepts any responsibility for the consequences of using it,
  or for whether it serves any particular purpose or works at all, unless he
  or she says so in writing.  Refer to the Aladdin Ghostscript Free Public
  License (the "License") for full details.
  
  Every copy of Aladdin Ghostscript must include a copy of the License,
  normally in a plain ASCII text file named PUBLIC.  The License grants you
  the right to copy, modify and redistribute Aladdin Ghostscript, but only
  under certain conditions described in the License.  Among other things, the
  License requires that the copyright notice and this notice be preserved on
  all copies.
*/

/* zfpngp.c */
/* PNG pixel predictor filter creation */
#include "ghost.h"
#include "errors.h"
#include "oper.h"
#include "idict.h"
#include "idparam.h"
#include "strimpl.h"
#include "spngpx.h"
#include "ifilter.h"

/* Common setup for encoding and decoding filters. */
private int
zpp_setup(os_ptr op, stream_PNGP_state *ppps)
{	int code, bpc;

	check_type(*op, t_dictionary);
	check_dict_read(*op);
	if ( (code = dict_int_param(op, "Colors", 1, 16, 1,
				    &ppps->Colors)) < 0 ||
	     (code = dict_int_param(op, "BitsPerComponent", 1, 16, 8,
				    &bpc)) < 0 ||	     
	     (bpc & (bpc - 1)) != 0 ||
	     (code = dict_uint_param(op, "Columns", 1, max_uint, 1,
				    &ppps->Columns)) < 0 ||
	     (code = dict_int_param(op, "Predictor", 10, 15, 15,
				    &ppps->Predictor)) < 0
	   )
	  return (code < 0 ? code : gs_note_error(e_rangecheck));
	ppps->BitsPerComponent = bpc;
	return 0;
}

/* <target> <dict> PNGPredictorEncode/filter <file> */
private int
zPNGPE(os_ptr op)
{	stream_PNGP_state pps;
	int code = zpp_setup(op, &pps);

	if ( code < 0 )
	  return code;
	return filter_write(op, 1, &s_PNGPE_template, (stream_state *)&pps, 0);
}

/* <source> <dict> PNGPredictorDecode/filter <file> */
private int
zPNGPD(os_ptr op)
{	stream_PNGP_state pps;
	int code = zpp_setup(op, &pps);

	if ( code < 0 )
	  return code;
	return filter_read(op, 1, &s_PNGPD_template, (stream_state *)&pps, 0);
}

/* ---------------- Initialization procedure ---------------- */

BEGIN_OP_DEFS(zfpngp_op_defs) {
		op_def_begin_filter(),
	{"2PNGPredictorDecode", zPNGPD},
	{"2PNGPredictorEncode", zPNGPE},
END_OP_DEFS(0) }
